import cjson
from infi.vendata.vmware_powertools.redis import RedisServer
from . import exceptions

class SnapshotStore(object):
    VM_SNAP_STORE = "vm_snaps"
    FOLDER_SNAP_STORE = "folder_snaps"
    DATASTORE_SNAP_STORE = "datastore_snaps"
    GROUP_STORE = "snap_groups"

    @classmethod
    def add_to_store(cls, store, key, snapshot):
        redis_client = RedisServer.get_client()
        snapshot_cjson = cjson.encode(snapshot)
        snapshot_id = snapshot["snapshot_id"]
        store = "{}_{}".format(store, key)
        redis_client.hset(store, snapshot_id, snapshot_cjson)

    @classmethod
    def get_from_store(cls, store, key):
        redis_client = RedisServer.get_client()
        store = "{}_{}".format(store, key)
        snapshot_list = redis_client.hgetall(store).values()
        snapshot_list = [cjson.decode(snapshot_json) for snapshot_json in snapshot_list]
        return snapshot_list

    @classmethod
    def del_from_store(cls, store, key, snapshot_id):
        redis_client = RedisServer.get_client()
        store = "{}_{}".format(store, key)
        redis_client.hdel(store, snapshot_id)

    @classmethod
    def get_snapshot_info(cls, store, key, snapshot_id):
        redis_client = RedisServer.get_client()
        store = "{}_{}".format(store, key)
        snapshot_json = redis_client.hget(store, snapshot_id)
        if snapshot_json is None:
            raise exceptions.SnapshotNotFound("Couldn't find snapshot id {}".format(snapshot_id))
        snapshot_info = cjson.decode(snapshot_json)
        return snapshot_info

    @classmethod
    def set_group_list(cls, group_uuid, group_list):
        redis_client = RedisServer.get_client()
        list_json = cjson.encode(group_list)
        redis_client.hset(cls.GROUP_STORE, group_uuid, list_json)

    @classmethod
    def get_group_list(cls, group_uuid):
        redis_client = RedisServer.get_client()
        list_json = redis_client.hget(cls.GROUP_STORE, group_uuid)
        if list_json is None:
            return []
        group_list = cjson.decode(list_json)
        return group_list

    @classmethod
    def del_group_list(cls, group_uuid):
        redis_client = RedisServer.get_client()
        redis_client.hdel(cls.GROUP_STORE, group_uuid)

    @classmethod
    def get_all_group_ids(cls):
        redis_client = RedisServer.get_client()
        return redis_client.hkeys(cls.GROUP_STORE)
