from infi.vendata.vmware_powertools import PROJECTROOT
from infi.vendata.powertools.utils import auth
from infi.vendata.credentials_store import base
import os


EXTENSION_CLIENT_USERNAME = "extension_client"


def safe_get_session_id():
    import cherrypy
    from infi.vendata.vmware_powertools.webapp.plugin.resource import get_session_id
    request = cherrypy.request
    try:
        return get_session_id(request)
    except:
        return None


def get_username_from_cherrypy():
    from infi.vendata.vmware_powertools.webapp.plugin.resource import get_session_id, ActiveSessions
    from infi.vendata.vmware_powertools.webapp.plugin.resource import get_extension_session_id
    import cherrypy
    request = cherrypy.request
    if request.handler is None:
        return EXTENSION_CLIENT_USERNAME
    session_id = get_session_id(request)
    if ActiveSessions.usernames.get(session_id):
        return ActiveSessions.usernames[session_id]
    elif session_id == get_extension_session_id(request):
        return EXTENSION_CLIENT_USERNAME
    raise cherrypy.HTTPError(400, "Session lost! Please restart your VMWare vSphere Client.")


def normalize_username(username):
    from md5 import md5
    # we add the newline so it'll be easy to match against `echo string | md5sum`
    return md5(username + '\n').hexdigest()


class VMwareCredentialsStore(base.FileCredentialsStore):
    """
    A credentials store for the VMware
    """
    def __init__(self):
        super(VMwareCredentialsStore, self).__init__()
        self.temp_credentials = {}

    def _load_credentials(self, serial):
        session_id = safe_get_session_id()
        username = normalize_username(self.get_username_from_cherrypy())
        return self.temp_credentials.get((username, session_id, str(serial))) or \
            super(VMwareCredentialsStore, self)._load_credentials(serial)

    def get_all_possible_credentials(self, serial):
        # evil workaround to avoid configuring VASA - try all available
        # credentials from all sessions in order to log in. VASA only needs read access
        from glob import glob
        from infi.vendata.credentials_store.base import Credentials
        serial = str(serial)
        for (username, session_id, stored_serial), credentials in self.temp_credentials.iteritems():
            if stored_serial in (serial, base.DEFAULT):
                yield credentials
        file_paths = glob(os.path.join(PROJECTROOT, "data", "credentials", "*"))
        for option in (serial, base.DEFAULT):
            for file_path in file_paths:
                data = self._load_file_by_path(file_path).get(option)
                if data:
                    yield Credentials.from_dict(data)

    def get_credentials(self, serial):
        # The GUI ensures that this is never called before there are credentials for this serial,
        # so no need to ask the user for credentials here
        credentials = self.get_existing_credentials(serial)
        if credentials:
            return credentials
        # We're not supposed to get here
        raise base.NoCredentialsException('Credentials are missing for system serial {}'.format(serial))

    def set_credentials(self, serial, credentials, remember_me=False):
        import cherrypy
        username = normalize_username(self.get_username_from_cherrypy())
        session_id = self.safe_get_session_id()
        self.temp_credentials[(username, session_id, str(serial))] = credentials
        if '' in cherrypy.tree.apps:
            cherrypy.tree.apps[''].root.plugin.views.host.clear_cache()
        if remember_me:
            super(VMwareCredentialsStore, self).set_credentials(serial, credentials)

    def reset_credentials(self):
        import cherrypy
        username = normalize_username(self.get_username_from_cherrypy())
        session_id = self.safe_get_session_id()
        for key in self.temp_credentials.keys():
            if key[0] == username and key[1] == session_id:
                self.temp_credentials.pop(key)
        if os.path.exists(self.get_file_path()):
            os.remove(self.get_file_path())
        cherrypy.tree.apps[''].root.plugin.views.host.clear_cache()

    def set_default_credentials(self, credentials, remember_me=False):
        self.set_credentials(base.DEFAULT, credentials, remember_me)

    def get_username_from_cherrypy(self):  # for easier monkey-patching
        return get_username_from_cherrypy()

    def safe_get_session_id(self):  # for easier monkey-patching
        return safe_get_session_id()

    def get_file_path(self):
        username = normalize_username(self.get_username_from_cherrypy())
        filepath = os.path.join(PROJECTROOT, "data", "credentials", "infinidat_credentials_{}".format(username))
        return filepath

    def set_credentials_for_extension_client(self, credentials, session_id=None):
        from mock import patch
        with patch.object(self, "get_username_from_cherrypy") as get_username_from_cherrypy:
            get_username_from_cherrypy.return_value = EXTENSION_CLIENT_USERNAME
            with patch.object(self, "safe_get_session_id") as safe_get_session_id:
                safe_get_session_id.return_value = session_id
                self.set_default_credentials(credentials)

auth.CREDENTIALS_STORE = VMwareCredentialsStore()
