from logging import getLogger
logger = getLogger(__name__)

def _get_all_running_tasks(client):
    from pyVmomi import vim
    from pyVmomi.SoapAdapter import ParserError
    filter_spec = vim.TaskFilterSpec(alarm=None, scheduledTask=None, state=["running", "queued"])
    collector = client.service_content.taskManager.CreateCollectorForTasks(filter_spec)
    while True:
        try:
            tasks = collector.ReadNextTasks(1)
        except ParserError:  # workaround for HIPVM-594
            logger.exception("failed to parse result of ReadNextTasks")
            continue
        if len(tasks) == 0:
            break
        for task in tasks:
            yield task
    collector.DestroyCollector()

def _get_extension_task_ids(client, extension_key):
    from itertools_recipes import flatten
    extension_list = client.service_content.extensionManager.extensionList
    my_extension_task_list = flatten([ext.taskList for ext in extension_list if ext.key == extension_key])
    return [task_type.taskID for task_type in my_extension_task_list]

def _cancel_stale_tasks(client, extension_key):
    from infi.pyvmomi_wrapper import Task
    my_extension_task_ids = _get_extension_task_ids(client, extension_key)
    for task in _get_all_running_tasks(client):
        if task.descriptionId in my_extension_task_ids:
            logger.info("Cancelling stale task, id %s", task.descriptionId)
            task = Task(client, task.task)
            task.set_state("error")

def cancel_stale_tasks():
    from infi.vendata.vmware_powertools.vcenter import vCenter
    from infi.vendata.vmware_powertools.config import Configuration
    logger.debug("Going over running tasks to look for stale tasks")
    extension_key = Configuration.get_vcenter_extension_key()
    client = vCenter.get_extension_client()
    _cancel_stale_tasks(client, extension_key)
    logger.debug("Stale tasks check done")
