from mock import patch


class vCenterLoginPatcher(object):
    """ used to patch vCenter to connect using passwords rather than OVF. Used by "--as-vcenter-xx" option """
    _username = None
    _password = None

    def __init__(self, vcenter_address, username, password):
        self._vcenter_address = vcenter_address
        self.__class__._username = username
        self.__class__._password = password
        self.patchers = []

    def __enter__(self):
        self._apply_monkey_patching()

    def _apply_monkey_patching(self):
        from infi.vendata.vmware_powertools.config import Configuration
        from infi.vendata.vmware_powertools.vcenter import vCenter
        patcher = patch.object(Configuration, "get_vcenter_address", return_value=self._vcenter_address)
        patcher.start()
        self.patchers.append(patcher)
        patcher = patch.object(vCenter, "get_vcenter_client", new=self.__class__.get_vcenter_client)
        patcher.start()
        self.patchers.append(patcher)
        patcher = patch.object(vCenter, "_send_post_request_to_vcenter", new=self.__class__._send_post_request_to_vcenter)
        patcher.start()
        self.patchers.append(patcher)
        patcher = patch.object(vCenter, "update_vm_version_if_necessary")
        patcher.start()
        self.patchers.append(patcher)

    def __exit__(self, *exc_info):
        exception_has_been_raised = exc_info != (None, None, None)
        self._stop_monkey_patching()
        if exception_has_been_raised:
            raise exc_info[0], exc_info[1], exc_info[2]

    def _stop_monkey_patching(self):
        [one_patch.stop() for one_patch in self.patchers]

    @classmethod
    def _send_post_request_to_vcenter(cls):
        from ..config import Configuration
        from datetime import datetime
        from infi.vendata.vmware_powertools.vcenter import vCenter
        client = vCenter.get_extension_client()
        try:
            client.login(cls._username, cls._password)
        except:
            return
        key = Configuration.get_vcenter_extension_key()
        if key not in [ext.key for ext in client.service_content.extensionManager.extensionList]:
            extension = vim.Extension(key=key, description=vim.Description(label="", summary=""), lastHeartbeatTime=datetime.fromtimestamp(0), version="")
            client.service_content.extensionManager.RegisterExtension(extension)

    @classmethod
    def get_vcenter_client(cls):
        from infi.pyvmomi_wrapper import Client
        from ..config import Configuration
        vcenter_address = Configuration.get_vcenter_address()
        client = Client(vcenter_address,
                        **Configuration.get_vcenter_extension_ssl_configuration())
        return client