import cherrypy
from ..resource import PluginResource


def get_vmware_url_params():
    from ....vcenter import vCenter
    client = vCenter.get_extension_client()
    session = client.session_manager.currentSession.key
    server = client.service_content.about.instanceUuid
    return '&'.join(['serverGuid={}'.format(server), 'locale=en_US',
                     'vimSessionKey={}'.format(session)])


def get_plugin_resources_dict():
    from ....vcenter import vCenter
    client = vCenter.get_extension_client()
    default_params = get_vmware_url_params()
    object_types = [("Clusters", "cluster", client.get_host_clusters()),
                    ("Hosts", "host", client.get_host_systems()),
                    ("VMs", "virtualmachine", client.get_virtual_machines()),
                    ("Datastores", "datastore", client.get_datastores()),
                    ("Folders", "folder", client.get_folders())]
    for object_name, object_uri, object_list in object_types:
        morefs = [(obj.name, client.get_reference_to_managed_object(obj)) for obj in object_list]
        params = '&'.join(['moref={}', default_params])
        uri = "/plugin/redirect/{}?{}".format(object_uri, params)
        uris = {moref: (uri.format(moref), name) for (name, moref) in morefs}
        yield object_name, uris


def get_plugin_logs_collect_url():
    return "/plugin/logs/collect?moref=Folder:group-d1&{}".format(get_vmware_url_params())


class Debug(PluginResource):
    @cherrypy.expose
    def index(self):
        self._check_disabled()
        resources = self._get_plugin_resources_dict()
        logs_collect = get_plugin_logs_collect_url()
        cherrypy.response.headers['Cache-Control'] = 'private, max-age=0, no-cache'
        return self.template_lookup.get_template('debug.mako').render(resources=resources, logs_collect=logs_collect)

    def _get_plugin_resources_dict(self):
        return list(get_plugin_resources_dict())

    @cherrypy.expose
    def refresh(self):
        return self.index()

    def _check_disabled(self):
        from ....config import Configuration
        # we use this trick to disable debug in production because there is no way to expose or unexpose the
        # class dynamically, and we can't decide whether to expose or not on startup
        if Configuration.is_in_production():
            raise cherrypy.HTTPRedirect("/static/html/404.html")
