import cherrypy
from ....resource import PluginResource
from infi.vendata.vmware_powertools.webapp import json_response
from infi.vendata.vmware_powertools.vcenter.shortcuts import moref_to_object
from infi.vendata.vmware_powertools import DATA_DIR
import os


class ReadinessException(Exception):
    pass


def is_host_ready(esx_host):
    from infi.vendata.vmware_powertools.mapping import is_host_connected
    from infi.vendata.vmware_powertools.vcenter.satp import InfinidatSatp
    if not is_host_connected(esx_host):
        raise ReadinessException("Host not connected")
    handler = InfinidatSatp(esx_host)
    if not handler.is_rule_installed():
        raise ReadinessException("InfiniBox claiming rule is not installed")
    if not handler.are_all_devices_with_correct_settings():
        raise ReadinessException("Not all devices have correct SATP and PSP setting")


def get_readiness_filepath(moref):
    return os.path.join(DATA_DIR, 'readiness', moref_to_object(moref).summary.hardware.uuid)


def is_host_previously_ready(moref):
    return os.path.exists(get_readiness_filepath(moref))


def mark_host_previously_ready(moref):
    filepath = get_readiness_filepath(moref)
    try:
        os.makedirs(os.path.dirname(filepath))
    except:
        pass
    open(filepath, 'a').close()


def fix_esx_host_readiness_state(esx_host):
    from infi.vendata.vmware_powertools.vcenter.satp import InfinidatSatp
    handler = InfinidatSatp(esx_host)
    handler.fix_everything()


class Readiness(PluginResource):

    @json_response
    def index(self, moref, silent, **kwargs):
        return getattr(self, cherrypy.request.method.upper())(moref, silent)

    def POST(self, moref, silent):
        fix_esx_host_readiness_state(moref_to_object(moref))
        mark_host_previously_ready(moref)

    def GET(self, moref, silent):
        # If silent is 'false', the check was initiated by the user. In this case we ignore the saved result.
        try:
            if silent == 'true':
                if is_host_previously_ready(moref):
                    return True
                return "not ready"
            is_host_ready(moref_to_object(moref))
            mark_host_previously_ready(moref)
            return True
        except ReadinessException, e:
            return str(e)
