from pyVmomi import vim


def _get_children(obj, motype):
    if isinstance(obj, vim.Datacenter):
        if motype == vim.HostSystem:
            return obj.hostFolder.childEntity
        if motype == vim.Folder:
            return [obj.vmFolder]
    if isinstance(obj, vim.ComputeResource):
        return obj.host
    if isinstance(obj, vim.Folder):
        return [child for child in obj.childEntity if isinstance(child, (vim.Folder, vim.Datacenter, motype))]
    if isinstance(obj, vim.HostSystem) and motype == vim.Datastore:
        return obj.datastore
    return []


def get_icon_name(obj):
    if isinstance(obj, (vim.Datacenter, vim.VirtualMachine, vim.Datastore, vim.ComputeResource, vim.HostSystem)):
        clsname = obj.__class__.__name__[4:].replace("ClusterComputeResource", "ComputeResource")
        return "/static/vmware/images/fancytree/{}.png".format(clsname)


def treeview_data(client, moref, motype_name, toplevel=False):
    obj = client.get_managed_object_by_reference(moref)
    motype = getattr(vim, motype_name)
    children = _get_children(obj, motype)
    res = [dict(title=child.name,
                lazy=len(_get_children(child, motype)) > 0,
                folder=isinstance(child, vim.Folder),
                data=dict(moref=client.get_reference_to_managed_object(child),
                          icon=get_icon_name(child),
                          activatable=isinstance(child, motype)))
           for child in children]
    if toplevel:
        res = [dict(title=obj.name,
                    expanded=True,
                    folder=isinstance(obj, vim.Folder),
                    children=res,
                    data=dict(activatable=False,
                              icon=get_icon_name(obj)))]
    return res
