var operations = {
    unblockUI: function() {
        $.unblockUI({fadeOut: 0});
    },

    goto_vm_callback: function(selElement, ajaxSource) {
        var callback = function() {
            var moref = selElement.attr('id')
            var url = ajaxSource + "&moref=" + moref + "&back_url=" + encodeURIComponent(window.location.href);
            window.location.replace(url);
        }
        return callback;
    },

    goto_dead_vm_callback: function(selElement, ajaxSource) {
        var callback = function() {
            var vm_uuid = selElement.attr('id')
            var infinibox_serial = selElement.data('serial')
            var infinibox_name = selElement.data('name')
            var vm_name = selElement.text()
            var url = ajaxSource + "&vm_name=" + vm_name +
                        "&vm_uuid=" + vm_uuid +
                        "&back_url=" + encodeURIComponent(window.location.href);
            window.location.replace(url);
        }
        return callback;
    },

    create_snapshot_popup_callback: function(selElement, ajaxSource) {
        var _callback = function() {
            var elem = $("#create-snap-modal");
            $(".snapname, .snapdesc", elem).val("");
            $("#create-snap-modal").dialog({
                modal: true,
                width: 540,
                height: 480,
                resizable: false,
                buttons: {
                    DONE: function() {
                        $(this).dialog("close");
                        operations.create_snapshot_callback();
                    },
                    CANCEL: function() {
                        $(this).dialog("close");
                    }
                }
            });
        };

        var callback = function() {
            // get the system serials of the checked vms
            selected = jQuery('.goto_vm', jQuery(":checked").parent().parent());
            infinibox_serials = jQuery.map(selected, function(element) {return jQuery(element).data("serial")});
            infinibox_serials = jQuery.unique(infinibox_serials);
            // create a recursive call stack: call "call_with_credentials" for every serial and eventually call
            // _callback, which is the actual popup function
            call_with_credentials_iterator = function() {
                if (infinibox_serials.length == 0) {
                    _callback();
                    return;
                }
                serial = infinibox_serials.pop();
                spinnerFactory.blockUI("huge");
                auth.call_with_credentials(serial, call_with_credentials_iterator, operations.unblockUI);
            };
            call_with_credentials_iterator();
        }
        return callback;
    },

    get_selected_vm_refs: function() {
        var vm_ref_list = "";
        var foreach = function(index, element) {
            if (jQuery(element).is(":checked")) {
                var vm_ref = jQuery('.goto_vm', jQuery(element).parent().parent()).attr("id");
                vm_ref_list += vm_ref + ";";
            }
        }
        jQuery('.vm_checkbox').each(foreach);
        return vm_ref_list;
    },

    create_snapshot_callback: function() {
        var elem = $("#create-snap-modal");
        var snap_name = $(".snapname", elem).val();
        var snap_desc = $(".snapdesc", elem).val();
        var consistent = $(".consistent", elem).is(":checked");
        var consolidate = $('.consolidate', elem).is(":checked");
        var vmrefs = operations.get_selected_vm_refs();
        var data = { vmrefs: vmrefs, snap_name: snap_name, snap_desc: snap_desc, consistent: consistent, consolidate: consolidate };
        var ajaxOptions = {type: "POST", dataType: "json", data: data,
                           cache: false, url: sources.operations.create_snapshot};
        spinnerFactory.blockUI("huge");
        view.ajax(ajaxOptions, null, null, operations.unblockUI);
    },

    setClickOnOperation: function(operationName, operationSources, parent) {
        var foreach = function(index, element) {
            var ajaxSource = operationSources[operationName]
            var selector = jQuery(element);
            var callback_getter = operations[operationName + "_callback"]
            var callback = callback_getter(selector, ajaxSource);
            selector.click(callback);
        }
        jQuery("." + operationName, parent).each(foreach);
    },

    init: function(operationSources) {
        operations.setClickOnOperation("goto_vm", operationSources);
        operations.setClickOnOperation("goto_dead_vm", operationSources);
        operations.setClickOnOperation("create_snapshot_popup", operationSources);
        init_actions_menu();
    }

}

var datastoreFolder = {
    checkboxChanged: function() {
        var noneChecked = true; // until proven otherwise
        jQuery(".vm_checkbox").each( function() {
            if (jQuery(this).is(":checked")) {
                noneChecked = false;    // proven otherwise
            }
        });
        if (noneChecked) {
            jQuery(".create_snapshot_popup").hide();
        } else {
            jQuery(".create_snapshot_popup").show();
        }
    },

    selectAll: function () {
        var newState = jQuery('#select-all').is(":checked");
        jQuery(".vm_checkbox").each( function() {
            jQuery(this).prop('checked', newState);
        });
        datastoreFolder.checkboxChanged();
    },

    initDataTable: function(sources) {
        var options = view.getDataTableOptions();
        var checkboxTitle = '<input type="checkbox" id="select-all" onclick="javascript:datastoreFolder.selectAll()"> Select All';
        options.ajax = sources.vm_table;
        options.searching = true;
        options.columns = [{ "title": "VM Name", "className": "table_name_cell", "width": "80%" },
                           { "title": checkboxTitle, "orderable": false, "className": "table_check_cell", "width": "20%" }];
        options.createdRow = function(row, data, dataIndex) {
            operations.setClickOnOperation("goto_vm", sources.operations, jQuery(row));
            operations.setClickOnOperation("goto_dead_vm", sources.operations, jQuery(row));
            jQuery(row, ".vm_checkbox").change(datastoreFolder.checkboxChanged);
        };
        options.language = {zeroRecords: "No Virtual Machines", search: "", loadingRecords: " ", processing: " "};
        var table = view.createDataTable($("#vmtable"), options);
    },

    getCallbackOnLoad: function(sources) {
        var callback = function() {
            jQuery(".expand").toggle();
            jQuery(".hidden").hide();
            datastoreFolder.initDataTable(sources);
            reload.init(sources.refresh);
            operations.init(sources.operations);
            jQuery(".dataTables_filter * input").attr("placeholder", "Search Here");
        }
        return callback;
    }
}
