var operations = {

    // Reload the datatable contating the target DOM element, and optionally call the callback afterwards
    reload_parent_table: function(target, callback) {
        var selInfinibox = target.closest(".infinibox.root");
        var dataTable = infinibox.getDataTable(selInfinibox);
        operations.reload_table(dataTable, callback);
    },

    reload_table: function(dataTable, callback) {
        view.blockElement(dataTable, true);
        var url = sources.datastore_table + "&serial=" + infinibox.getSerial(dataTable.closest(".infinibox.root"));
        done_callback = function(data) {
            dataTable.fnClearTable();
            var actions_template = _.template($('#datastore_actions_template').html());
            _.each(data.return_value, function(datastore_info) {
                var ds_div = $('<div/>').text(datastore_info.datastore_name);
                if (!datastore_info.mounted) {
                    var new_div = $("<div/>");
                    new_div.text(ds_div.text() + " (unmounted)");
                    ds_div.text("");
                    new_div.attr("class", "inaccessible");
                    ds_div.append(new_div);
                }
                var vol_link = $('<a/>')
                    .attr('href', datastore_info.volume_url)
                    .attr('target', '_blank')
                    .text(datastore_info.volume_name);
                var row = {
                    datastore:  ds_div[0].innerHTML,
                    name:       vol_link[0].outerHTML,
                    size:       create_vol_modal.format_size(datastore_info.volume_size),
                    paths:      datastore_info.paths || datastore_info.hosts,
                    lun:        datastore_info.lun,
                    device:     datastore_info.device_name,
                    actions:    actions_template(datastore_info)
                };
                dataTable.fnAddData(row);
            });
            if (callback) callback();
        },
        view.ajax(
            {dataType: 'json', type: 'POST', url: url},
            done_callback,
            null,
            function() {
                view.blockElement(dataTable, false);
            }
        );
    },

    popup_set_credentials: function(event) {
        event.preventDefault();
        var selElement = $(event.target);
        var serial = selElement.closest(".infinibox.root").data("serial");
        auth.ask_credentials(serial, null, "SET CREDENTIALS");
    },

    perform_operation: function(selElement, ajaxSource, data, done_callback) {
        var selInfinibox = selElement.closest('.infinibox.root');
        var system_serial = selInfinibox.data('serial');
        view.blockElement(selInfinibox, true);
        auth.call_with_credentials(system_serial, function() {
            data = data || {};
            data.serial = data.serial || selElement.closest('.actions').data("serial");
            var selInfinibox = selElement.closest('.infinibox.root');
            var dataTable = infinibox.getDataTable(selInfinibox);
            var ajaxOptions = {type: "POST", dataType: "json", data: data, cache: false, url: ajaxSource};
            view.ajax(ajaxOptions,
                      function() {operations.reload_table(dataTable, done_callback)},
                      function() {},
                      function() {view.blockElement(selInfinibox, false)});
        });
    },

    unmap: function(event) {
        if (entity_name == 'datacenter') {
            return operations.unmap_from_datacenter(event);
        }
        var volume_name = $(event.target).closest(".actions").data("name");
        show_areyousure("CONFIRM", 'Unmap volume "' + volume_name + '"?', 'This can cause service interruption or data loss.', 'UNMAP', function() {
            var selElement = $(event.target);
            var system_serial = selElement.closest('.infinibox.root').data('serial');
            operations.perform_operation(selElement, sources.operations.unmap);
        });
    },

    delete_vol: function(event) {
        var volume_name = $(event.target).closest(".actions").data("name");
        show_areyousure("CONFIRM", 'Delete volume "' + volume_name + '"?', 'This can cause service interruption or data loss.', 'DELETE', function() {
            var selElement = $(event.target);
            var system_serial = selElement.closest('.infinibox.root').data('serial');
            operations.perform_operation(selElement, sources.operations.delete_vol);
        });
    },

    unmap_from_datacenter: function(event) {
        var selElement = $(event.target);
        var selInfinibox = selElement.closest('.infinibox.root');
        view.blockElement(selInfinibox, true);
        auth.call_with_credentials(selInfinibox.data('serial'),
            function() {
                var dataTable = infinibox.getDataTable(selInfinibox);
                view.blockElement(selInfinibox, true);
                var serial = selElement.closest('.actions').data('serial');
                ajaxOptions = {dataType: 'json', type: 'POST', url: sources.operations.get_mappings, data: {serial: serial}}
                done_callback = function(data) {
                    view.blockElement(selInfinibox, false);
                    var html = _.template($('#unmap_from_datacenter_template').html(), data.return_value);
                    $(html).dialog({
                        modal: true,
                        width: 540,
                        height: 510,
                        resizable: false,
                        buttons: {
                            OK: function() {
                                var morefs = $('#unmap-from-datacenter-list input:checked').map(function() {
                                    return this.value;
                                }).get();
                                if (morefs.length == 0) {
                                    showError("ERROR", "No clusters or hosts were selected.");
                                    return;
                                }
                                $(this).dialog("destroy");
                                operations.perform_operation(selElement, sources.operations.unmap, {unmap_morefs: morefs});
                            },
                            CANCEL: function() {
                                $(this).dialog("destroy");
                            }
                        }
                    },
                    function() {
                        view.blockElement(selInfinibox, false);
                    });
                }
                view.ajax(ajaxOptions, done_callback);
            },
            function() {
                view.blockElement(selInfinibox, false);
            }
        );
    },

    create_filesystem: function(event) {
        var selElement = $(event.target);
        operations.perform_operation(selElement, sources.operations.create_filesystem);
    },

    resize: function(event) {
        resize_check = function(old_size, new_size, units) {
            if (units == "GB") new_size = new_size * 1024 * 1024 * 1024;
            if (units == "TB") new_size = new_size * 1024 * 1024 * 1024 * 1024;
            if (new_size < old_size) {
                showError("ERROR", "Cannot shrink datastore");
                return false;
            }
            if (new_size <= old_size + (0.3 * 1024 * 1024 * 1024)) {
                showError("ERROR", "New size must be at least 0.3 GB larger than the old size");
                return false;
            }
            return true;
        }
        resize_modal.popup(event, resize_check);
    },

    rename: function(event) {
        rename_modal.popup(event);
    },

    rescan: function(event) {
        var ajaxOptions = {cache: false, url: sources.operations.rescan};
        spinnerFactory.blockUI("huge");
        view.ajax(ajaxOptions, function() {
            var ajaxOptions = {cache: false, url: sources.refresh};
            view.ajax(ajaxOptions, reload.reload, null, null);
        }, null, function() {
            $.unblockUI({fadeOut: 0});
        });
    },

    init: function(operationSources) {
        create_vol_modal.init();
        tooltip.create($('#capacity-tooltip'), "Capcity units are in powers of 1024 bytes");
        $("#vol-modal-count").regexRestrict(RegExp("[^0-9]", "g"));
        map_vol_modal.init(operationSources);
        $(".set-credentials").click(operations.popup_set_credentials);
        init_actions_menu();
        operations.init_action_listeners();
    },

    init_action_listeners: function() {
        $('body')
            .on('click', '.unmap', operations.unmap)
            .on('click', '.create_filesystem', operations.create_filesystem)
            .on('click', '.resize', operations.resize)
            .on('click', '.delete_vol', operations.delete_vol)
            .on('click', '.rename', operations.rename);
    }

}

var hostCluster = {
    initDataTable: function(sources) {
        var foreach = function(index, tableInfinibox) {
            var selector = $(tableInfinibox);
            var options = view.getDataTableOptions();
            if (!infinibox.isSupported(selector.parents(".infinibox.root"))) {
                return;
            }
            options.language = {zeroRecords: "No Mapped Volumes", search: "", loadingRecords: " ", processing: " "};
            options.dom = "C" + options.dom;    // ColVis
            options.colVis = {
                align: "right",
                buttonText: " ",
                exclude: [0, 6]
            };
            options.columns = [
                {name: 'datastore', data: 'datastore'},
                {name: 'name',      data: 'name'},
                {name: 'size',      data: 'size',    type: "size",    width: "60px"},
                {name: 'paths',     data: 'paths',   type: "numeric", width: "80px"},
                {name: 'lun',       data: 'lun',     type: "numeric", width: "50px"},
                {name: 'device',    data: 'device'},
                {name: 'actions',   data: 'actions', width: "192px",  orderable: false}
            ];
            var table = view.createDataTable(selector, options);
            // hack to prevent sZeroRecords from displaying until loading is done
            table.api().row.add({
                datastore:  ' ',
                name:       ' ',
                size:       ' ',
                paths:      ' ',
                lun:        ' ',
                device:     ' ',
                actions:    ' '
            });
            table.api().draw();
            operations.reload_table(table);
        }
        $(".infinibox.table").each(foreach);
    },

    getCallbackOnLoad: function(sources) {
        var callback = function() {
            $.ajaxSetup({timeout: 1000*60*10});
            readiness.init(sources.status);
            registration.init(sources.registration);
            actions.initToggle();
            hostCluster.initDataTable(sources);
            $(".hidden").hide();
            operations.init(sources.operations);
            reload.init(sources.refresh);
            if ($(".infinibox.action-toggler").size() == 1)
                view.select(".infinibox.action-toggler").click();
        }
        return callback;
    }
}
