from logging import getLogger
logger = getLogger(__name__)


class WebServer(object):
    _webapps = []
    _vasa_provider = None

    @classmethod
    def start(cls, with_vasa_provider=True):
        cls._start_webapp()
        if with_vasa_provider:
            cls._start_vasa_provider()

    @classmethod
    def _start_vasa_provider(cls):
        from gevent.server import StreamServer
        from ..vasa_provider.zsi.zsi_wrapper import ServiceContainer
        from ..config import Configuration
        ssl_config = Configuration.get_webserver_ssl_configuration()
        address, port = Configuration.get_vasa_provider_bind_address_and_port()
        cls._vasa_provider = StreamServer((address, port), ServiceContainer().process_request, **ssl_config)
        cls._vasa_provider.start()


    @classmethod
    def _start_webapp(cls):
        from gevent.pywsgi import WSGIServer
        from ..config import Configuration
        from ..webapp import WebApplication
        from ..logging import log_timing, logbook_tag
        from gevent.socket import error as error_class
        from errno import EADDRINUSE
        from infi.vendata.vmware_powertools.utils import find_and_kill_listening_process

        socket_args = Configuration.get_webapp_bind_address_and_port()
        ssl_socket_args = Configuration.get_webapp_bind_address_and_ssl_port()
        ssl_config = Configuration.get_webserver_ssl_configuration()
        wsgi_app = WebApplication.get_wsgi_application()
        WSGIServer.handler_class.handle_one_response = logbook_tag(log_timing(WSGIServer.handler_class.handle_one_response))
        cls._webapps = [WSGIServer(socket_args, wsgi_app, log=None),
                        WSGIServer(ssl_socket_args, wsgi_app, log=None, **ssl_config)]
        try:
            for webapp in cls._webapps:
                webapp.start()
        except error_class, error_instance:
            if error_instance.errno == EADDRINUSE:
                logger.warn("address already in use, going to kill some processes and then suicide")
                find_and_kill_listening_process(webapp.address[1])
                logger.warn("and now we die")
                raise SystemExit(1)
            raise
        logger.info("Started WSGI server")

    @classmethod
    def stop(cls, with_vasa_provider=True):
        logger.info("Stopped WSGI server")
        list([webapp.stop() for webapp in cls._webapps if webapp is not None])
        if with_vasa_provider and cls._vasa_provider is not None:
            cls._vasa_provider.stop()

    @classmethod
    def serve_forever(cls):
        try:
            list([webapp._stop_event.wait() for webapp in cls._webapps])
        except KeyboardInterrupt:
            pass
        finally:
            cls.stop()
