#!/bin/sh
TARGETDIR=${1:-.}
HOSTNAME=`uname -n`
OPENSSL_CONF_DIR=/opt/vmware/etc/ssl
OPENSSL_CONF=$OPENSSL_CONF_DIR/openssl.conf
TMP_DIR=/var/tmp/tmp-certs

trap "rm -rf $TMP_DIR; exit 0" 0 1 2 15

echo "Generating SSL certificates for sfcb in $TARGETDIR"
logger -- "Generating SSL certificates for sfcb in $TARGETDIR"

if [ ! -f $OPENSSL_CONF ]
then
    echo "WARNING: Unable to find openssl configuration file, using default"
    mkdir -p $OPENSSL_CONF_DIR
    cat > $OPENSSL_CONF <<EOF
[req]
distinguished_name=user_dn
prompt=no

[user_dn]
CN=WARNING: Config file not found
EOF
fi

#openssl tools uses path in RANDFILE to write out a seeding file, this fixes 
#the "Unable to to write 'random state' message during key gen
RANDFILE=/root
export RANDFILE

mkdir -p $TMP_DIR;
chown root:root $TMP_DIR;
chmod 700 $TMP_DIR;

umask 0277; openssl req -x509 -days 10000 -newkey rsa:2048 \
   -nodes -config $OPENSSL_CONF   \
   -keyout $TMP_DIR/key.pem -out $TMP_DIR/cert.pem > /dev/null 2>&1

# TODO: Currently, we cannot change the permission of sfcb certs from 444 to 400
# because vamicli and other cli's need these certs, otherwise they won't work 
# for users other than root.

# server certs
cp $TMP_DIR/cert.pem $TARGETDIR/server.pem
cp $TMP_DIR/key.pem $TARGETDIR/file.pem
chmod 444 $TARGETDIR/server.pem $TARGETDIR/file.pem

# client certs
cp $TMP_DIR/cert.pem $TARGETDIR/client.pem
chmod 444 $TARGETDIR/client.pem
